//------------------------------------------------------------------------------------------------------------------------------------------------
//  Modded by GB to record two emonTX and three CT inputs from each emonTX

// emonGLCD Home Energy Monitor example
// to be used with nanode Home Energy Monitor example

// Uses power1 variable - change as required if your using different ports

// emonGLCD documentation http://openEnergyMonitor.org/emon/emonglcd

// RTC to reset Kwh counters at midnight is implemented is software. 
// Correct time is updated via NanodeRF which gets time from internet
// Temperature recorded on the emonglcd is also sent to the NanodeRF for online graphing

// GLCD library by Jean-Claude Wippler: JeeLabs.org
// 2010-05-28 <jcw@equi4.com> http://opensource.org/licenses/mit-license.php
//
// Authors: Glyn Hudson and Trystan Lea
// Part of the: openenergymonitor.org project
// Licenced under GNU GPL V3
// http://openenergymonitor.org/emon/license

// THIS SKETCH REQUIRES:

// Libraries in the standard arduino libraries folder:
//
//	- OneWire library	http://www.pjrc.com/teensy/td_libs_OneWire.html
//	- DallasTemperature	http://download.milesburton.com/Arduino/MaximTemperature
//                           or https://github.com/milesburton/Arduino-Temperature-Control-Library
//	- JeeLib		https://github.com/jcw/jeelib
//	- RTClib		https://github.com/jcw/rtclib
//	- GLCD_ST7565		https://github.com/jcw/glcdlib
//
// Other files in project directory (should appear in the arduino tabs above)
//	- icons.ino
//	- templates.ino
//
//-------------------------------------------------------------------------------------------------------------------------------------------------

#include <JeeLib.h>
#include <GLCD_ST7565.h>
#include <avr/pgmspace.h>
GLCD_ST7565 glcd;

#include <OneWire.h>		    // http://www.pjrc.com/teensy/td_libs_OneWire.html
#include <DallasTemperature.h>      // http://download.milesburton.com/Arduino/MaximTemperature/ (3.7.2 Beta needed for Arduino 1.0)

#include <RTClib.h>                 // Real time clock (RTC) - used for software RTC to reset kWh counters at midnight
#include <Wire.h>                   // Part of Arduino libraries - needed for RTClib
RTC_Millis RTC;

//--------------------------------------------------------------------------------------------
// RFM12B Settings
//--------------------------------------------------------------------------------------------
#define MYNODE 20            // Should be unique on network, node ID 30 reserved for base station
#define freq RF12_868MHZ     // frequency - match to same frequency as RFM12B module (change to 868Mhz or 915Mhz if appropriate)
#define group 210 

#define ONE_WIRE_BUS 5              // temperature sensor connection - hard wired 

unsigned long fast_update, slow_update;

OneWire oneWire(ONE_WIRE_BUS);
DallasTemperature sensors(&oneWire);
double temp,maxtemp,mintemp;

//---------------------------------------------------
// Data structures for transfering data between units
//---------------------------------------------------
typedef struct { int power1, power2, power3, Vrms, nodetemp; } PayloadTXa;         // neat way of packaging data for RF comms
PayloadTXa emontxa;

typedef struct { int power1, power2, power3, Vrms, nodetemp; } PayloadTXb;         // neat way of packaging data for RF comms
PayloadTXb emontxb;

typedef struct { int temperature; } PayloadGLCD;
PayloadGLCD emonglcd;

int hour = 0, minute = 0, second = 0;
double cta1kwh = 0;
double cta2kwh = 0;
double cta3kwh = 0;
double nodeatemp,nodeamaxtemp,nodeamintemp;

double ctb1kwh = 0;
double ctb2kwh = 0;
double ctb3kwh = 0;
double nodebtemp,nodebmaxtemp,nodebmintemp;

// Solar PV Variables
const int maxgen=10000;              // peak output of soalr PV system in W - used to calculate when to change cloud icon to a sun
const int PV_gen_offset=0;         // When generation drops below this level generation will be set to zero - used to force generation level to zero at night
int cval_gen;
int cval_use;

const int LDRpin=4;    		    // analog pin of onboard lightsensor 
int cval_cta1;
int cval_cta2;
int cval_cta3;

int cval_ctb1;
int cval_ctb2;
int cval_ctb3;

//-------------------------------------------------------------------------------------------- 
// Flow control
//-------------------------------------------------------------------------------------------- 
unsigned long last_emontxa;                   // Used to count time from last emontx update 1st emonTX assigned as node 10
unsigned long last_emontxb;                   // Used to count time from last emontx update 2nd emonTX assigned as node 11
unsigned long last_emonbase;                  // Used to count time from last emonbase update assigned as node 15

//--------------------------------------------------------------------------------------------
// Setup
//--------------------------------------------------------------------------------------------
void setup()
{
  rf12_initialize(MYNODE, freq,group);
  glcd.begin(0x20);
  glcd.backLight(200);
  
  sensors.begin();                         // start up the DS18B20 temp sensor onboard  
  sensors.requestTemperatures();
  temp = (sensors.getTempCByIndex(0));     // get inital temperture reading
  mintemp = temp; maxtemp = temp;          // reset min and max
  
  nodeamintemp = nodeatemp; nodeamaxtemp = nodeatemp;
  nodebmintemp = nodebtemp; nodebmaxtemp = nodebtemp;
  
}

//--------------------------------------------------------------------------------------------
// Loop
//--------------------------------------------------------------------------------------------
void loop()
{
  
  if (rf12_recvDone())
  {
    if (rf12_crc == 0 && (rf12_hdr & RF12_HDR_CTL) == 0)  // and no rf errors
    {
      int node_id = (rf12_hdr & 0x1F);
      if (node_id == 10) {emontxa = *(PayloadTXa*) rf12_data; last_emontxa = millis();} 
      if (node_id == 11) {emontxb = *(PayloadTXb*) rf12_data; last_emontxb = millis();}
      
      if (node_id == 15)
      {
        RTC.adjust(DateTime(2012, 1, 1, rf12_data[1], rf12_data[2], rf12_data[3]));
        last_emonbase = millis();
      } 
    }
  }

  //--------------------------------------------------------------------------------------------
  // Display update every 200ms
  //--------------------------------------------------------------------------------------------
  if ((millis()-fast_update)>200)
  {
    fast_update = millis();
    
    DateTime now = RTC.now();
    int last_hour = hour;
    hour = now.hour();
    minute = now.minute();
    second = now.second();

    cta1kwh += (emontxa.power1 * 0.2) / 3600000;
    cval_cta1 = cval_cta1 + (emontxa.power1 - cval_cta1)*0.50;

    cta2kwh += (emontxa.power2 * 0.2) / 3600000;
    cval_cta2 = cval_cta2 + (emontxa.power2 - cval_cta2)*0.50;

    // Use below if using CT3 display as import / export function
    cval_use = cval_cta1;                                    //ct1 is measuring consumption
    cval_gen = cval_cta2;                                    //ct2 is measuring generation
    cval_cta3 = cval_use - cval_gen;                         //set ct3 to display import or export power
	

    nodeatemp = (emontxa.nodetemp);
    if (nodeatemp > 100) nodeatemp = 99.9;
    if (nodeatemp < -100) nodeatemp = -99.9;
    if (nodeatemp > nodeamaxtemp) nodeamaxtemp = nodeatemp;
    if (nodeatemp < nodeamintemp) nodeamintemp = nodeatemp;

    ctb1kwh += (emontxb.power1 * 0.2) / 3600000;
    cval_ctb1 = cval_ctb1 + (emontxb.power1 - cval_ctb1)*0.50;

    ctb2kwh += (emontxb.power2 * 0.2) / 3600000;
    cval_ctb2 = cval_ctb2 + (emontxb.power2 - cval_ctb2)*0.50;

    ctb3kwh += (emontxb.power3 * 0.2) / 3600000;
    cval_ctb3 = cval_ctb3 + (emontxb.power3 - cval_ctb3)*0.50;

    nodebtemp = (emontxb.nodetemp);
    if (nodebtemp > 100) nodebtemp = 99.9;
    if (nodebtemp < -100) nodebtemp = -99.9;
    if (nodebtemp > nodebmaxtemp) nodebmaxtemp = nodebtemp;
    if (nodebtemp < nodebmintemp) nodebmintemp = nodebtemp;

    draw_genny_page(cval_cta1, cta1kwh, cval_cta2, cta2kwh, cval_cta3, cta3kwh, nodeatemp, nodeamintemp, nodeamaxtemp, cval_ctb1, ctb1kwh, cval_ctb2, ctb2kwh, cval_ctb3, ctb3kwh, nodebtemp, nodebmintemp, nodebmaxtemp, temp, mintemp, maxtemp, hour, minute, second, last_emontxa, last_emontxb, last_emonbase);
    glcd.refresh();
	
    glcd.backLight(255);


  } 
  
  if ((millis()-slow_update)>10000)
  {
    slow_update = millis();

    sensors.requestTemperatures();
    temp = (sensors.getTempCByIndex(0));
    if (temp > maxtemp) maxtemp = temp;
    if (temp < mintemp) mintemp = temp;
   
    emonglcd.temperature = (int) (temp * 100);                          // set emonglcd payload
    int i = 0; while (!rf12_canSend() && i<10) {rf12_recvDone(); i++;}  // if ready to send + exit loop if it gets stuck as it seems too
    rf12_sendStart(0, &emonglcd, sizeof emonglcd);                      // send emonglcd data
    rf12_sendWait(0);    
  }
}